#!/usr/bin/python3

import requests
import json
import searchguard.settings as settings
from searchguard.exceptions import RoleMappingException, CheckRoleMappingExistsException, ViewRoleMappingException, \
    DeleteRoleMappingException, CreateRoleMappingException, ModifyRoleMappingException, CheckRoleExistsException, \
    ViewAllRoleMappingException
from searchguard.roles import check_role_exists


PROPERTIES_KEYS = {"users", "backendroles", "hosts"}


def _send_api_request(role, properties):
    """Private function to process API calls for the rolemapping module"""
    create_sg_rolemapping = requests.put('{}/rolesmapping/{}'.format(settings.SEARCHGUARD_API_URL, role),
                                         data=json.dumps(properties),
                                         headers=settings.HEADER,
                                         auth=settings.SEARCHGUARD_API_AUTH)

    if create_sg_rolemapping.status_code in (200, 201):
        # Role mapping created or updated successfully
        return

    # Error when creating/updating the role mapping
    raise RoleMappingException('Error creating/updating the mapping for role {} - msg {}'.format(
        role, create_sg_rolemapping.text))


def check_rolemapping_exists(role):
    """Returns True of False depending on whether the requested role mapping exists in Search Guard"""
    rolemapping_exists_check = requests.get('{}/rolesmapping/{}'.format(settings.SEARCHGUARD_API_URL, role),
                                            auth=settings.SEARCHGUARD_API_AUTH)

    if rolemapping_exists_check.status_code == 200:
        # Role mapping exists in SearchGuard
        return True
    elif rolemapping_exists_check.status_code == 404:
        # Role mapping does not exist in SearchGuard
        return False
    else:
        # Could not fetch valid output
        raise CheckRoleMappingExistsException('Unknown error checking whether role mapping for {} exists'.format(role))


def view_all_rolemappings():
    """Returns the properties for the requested role mappings if it exists"""
    view_all_sg_rolemapping = requests.get('{}/rolesmapping/'.format(settings.SEARCHGUARD_API_URL),
                                           auth=settings.SEARCHGUARD_API_AUTH)

    if view_all_sg_rolemapping.status_code == 200:
        return json.loads(view_all_sg_rolemapping.text)
    else:
        # Could not fetch valid output
        raise ViewAllRoleMappingException('Unknown error retrieving all role mappings')


def view_rolemapping(role):
    """Returns the properties for the requested role mapping if it exists"""
    view_sg_rolemapping = requests.get('{}/rolesmapping/{}'.format(settings.SEARCHGUARD_API_URL, role),
                                       auth=settings.SEARCHGUARD_API_AUTH)

    if view_sg_rolemapping.status_code == 200:
        return json.loads(view_sg_rolemapping.text)
    elif view_sg_rolemapping.status_code == 404:
        # Raise exception because the role mapping does not exist
        raise ViewRoleMappingException('Error viewing the role mapping for {}, does not exist'.format(role))
    else:
        # Could not fetch valid output
        raise ViewRoleMappingException('Unknown error checking whether role mapping for {} exists'.format(role))


def delete_rolemapping(role):
    """Deletes a Search Guard role mapping. Returns when successfully deleted"""
    if check_rolemapping_exists(role):
        # The role mapping does exist, let's delete it
        delete_sg_rolemapping = requests.delete('{}/rolesmapping/{}'.format(settings.SEARCHGUARD_API_URL, role),
                                                auth=settings.SEARCHGUARD_API_AUTH)

        if delete_sg_rolemapping.status_code == 200:
            # Role mapping deleted successfully
            return
        else:
            # Raise exception because we could not delete the role mapping
            raise DeleteRoleMappingException('Error deleting the role mapping for role {} '
                                             '- msg: {}'.format(role, delete_sg_rolemapping.text))
    else:
        # Raise exception because the role mapping does not exist
        raise DeleteRoleMappingException('Error deleting the role mapping for role {}, does not exist'.format(role))


def create_rolemapping(role, properties):
    """Creates a Search Guard role mapping. Returns when successfully created
    It is required to specify at least one of: users, backendroles or hosts in the properties argument.
    We do not use the PATCH endpoint for backwards compatibility with Elasticsearch before 6.4.0

    :param str role: Name of the role mapping to create in Search Guard
    :param dict properties: Search Guard role mapping fields (users, backendroles and/or hosts)
    :raises: CreateRoleMappingException, CheckRoleExistsException
    """
    if not check_role_exists(role):
        raise CheckRoleExistsException('Role {} does not exist'.format(role))

    if not any(key in properties for key in PROPERTIES_KEYS):
        # Raise exception because we did not receive valid properties
        raise CreateRoleMappingException('Error creating mapping for role {} - Include at least one of: users, '
                                         'backendroles or hosts keys in the properties argument'.format(role))

    _send_api_request(role, properties)
    return


def modify_rolemapping(role, properties, action="replace"):
    """Modifies a Search Guard role mapping. Returns when successfully modified
    It is required to specify at least one of: users, backendroles or hosts in the properties argument.
    We do not use the PATCH endpoint for backwards compatibility with Elasticsearch before 6.4.0

    :param str role: Name of the role mapping to create in Search Guard
    :param dict properties: Search Guard role mapping fields (users, backendroles and/or hosts)
    :param str action: Defines what to do with the properties. Defaults to replace (overwrites existing
    properties). Other options are merge (combine the properties with existing ones) or split
    (removes the properties from existing ones)
    :raises: ModifyRoleMappingException
    """
    if not check_rolemapping_exists(role):
        raise ModifyRoleMappingException('Mapping for role {} does not exist'.format(role))

    if not any(key in properties for key in PROPERTIES_KEYS):
        # Raise exception because we did not receive valid properties
        raise ValueError('Error modifying mapping for role {} - Include at least one of: users, '
                         'backendroles or hosts keys in the properties argument'.format(role))

    # Retrieve existing properties of the role mapping:
    rolemapping = view_rolemapping(role)
    for property in PROPERTIES_KEYS:
        if property not in rolemapping[role]:
            rolemapping[role][property] = list()

    if action is "merge":
        # Merge the requested properties with existing properties in the role mapping.

        rolemapping[role]['users'] = \
            sorted(set(rolemapping[role]['users'] + properties.get('users', [])))
        rolemapping[role]['backendroles'] = \
            sorted(set(rolemapping[role]['backendroles'] + properties.get('backendroles', [])))
        rolemapping[role]['hosts'] = \
            sorted(set(rolemapping[role]['hosts'] + properties.get('hosts', [])))

        _send_api_request(role, rolemapping[role])
        return

    if action is "split":
        # Remove the requested properties from existing properties in the role mapping.

        rolemapping[role]['users'] = [item for item in rolemapping[role]['users']
                                      if item not in properties['users']]
        rolemapping[role]['backendroles'] = [item for item in rolemapping[role]['backendroles']
                                             if item not in properties['backendroles']]
        rolemapping[role]['hosts'] = [item for item in rolemapping[role]['hosts']
                                      if item not in properties['hosts']]

        _send_api_request(role, rolemapping[role])
        return

    # No merge or split action, overwrite existing properties:
    _send_api_request(role, properties)


def list_rolemappings_for_user(user, roles=None, skip_missing_roles=False):
    """Get list of rolemappings that contain the given user. It is possible to add a list of roles to check.
    If no list is added, all rolemappings are evaluated. Non-existent roles can be excluded.

    :param str user: Name of user
    :param list roles: List of rolemappings to be checked for the given user
    :param bool skip_missing_roles: Skip missing roles or throw ViewRoleMappingException
    :returns list: list of rolemappings with the given user
    :raises: ViewRoleMappingException
    """
    if roles:
        if skip_missing_roles:
            user_rolemappings = list()

            for role in roles:
                try:
                    if user in view_rolemapping(role)[role]['users']:
                        user_rolemappings.append(role)
                except ViewRoleMappingException:
                    pass
        else:
            user_rolemappings = [role for role in roles if user in view_rolemapping(role)[role]['users']]

    else:
        user_rolemappings = [r for r, p in view_all_rolemappings().items() if user in p['users']]

    return sorted(set(user_rolemappings))



class SearchGuardException(Exception):
    pass


class CreateUserException(SearchGuardException):
    pass


class UserAlreadyExistsException(CreateUserException):
    pass


class DeleteUserException(SearchGuardException):
    pass


class CreateRoleException(SearchGuardException):
    pass


class RoleAlreadyExistsException(CreateRoleException):
    pass


class DeleteRoleException(SearchGuardException):
    pass


class ViewRoleException(SearchGuardException):
    pass


class ViewUserException(SearchGuardException):
    pass


class ModifyUserException(SearchGuardException):
    pass


class ModifyRoleException(SearchGuardException):
    pass


class ListUsersException(SearchGuardException):
    pass


class CheckUserExistsException(SearchGuardException):
    pass


class CheckRoleExistsException(SearchGuardException):
    pass


class RoleMappingException(SearchGuardException):
    pass


class CreateRoleMappingException(SearchGuardException):
    pass


class CheckRoleMappingExistsException(SearchGuardException):
    pass


class ModifyRoleMappingException(SearchGuardException):
    pass


class ViewAllRoleMappingException(SearchGuardException):
    pass


class ViewRoleMappingException(SearchGuardException):
    pass


class DeleteRoleMappingException(SearchGuardException):
    pass



import os

HEADER = {'content-type': 'application/json'}
SEARCHGUARD_API_URL = os.environ.get('SEARCHGUARD_API_URL', '')
SEARCHGUARD_API_AUTH = (os.environ.get('SEARCHGUARD_API_USER', ''), os.environ.get('SEARCHGUARD_API_PASS', ''))



#!/usr/bin/python3

import requests
import warnings
import json
import random
import string
import searchguard.settings as settings
from searchguard.exceptions import *


def password_generator(size=25, chars=string.ascii_uppercase + string.ascii_lowercase + string.digits):
    """Returns a random 25 character password"""
    return ''.join(random.choice(chars) for _ in range(size))


def check_user_exists(username):
    """Returns True of False depending on whether the requested user exists in Search Guard"""
    user_exists_check = requests.get('{}/internalusers/{}'.format(settings.SEARCHGUARD_API_URL, username), auth=settings.SEARCHGUARD_API_AUTH)

    if user_exists_check.status_code == 200:
        # Username exists in SearchGuard
        return True
    elif user_exists_check.status_code == 404:
        # Username does not exist in SearchGuard
        return False
    else:
        # Could not fetch valid output
        raise CheckUserExistsException('Unknown error checking whether user {} exists'.format(username))


def create_user(username, password=None, properties=None):
    """Creates a new Search Guard user and returns the generated password

    :param str username: the username
    :param str password: the password (by default will generate a password)
    :param dict properties: dict of user properties (roles, attributes, hash, etc), matching
    the API request body. Following the API, if "hash" is specified the password
    won't be used, then the return value is empty string.
    If password is passed both explicitly and as a property,
    mismatching passwords would raise a ValueError.

    :raises: UserAlreadyExistsException, CreateUserException, ValueError
    :return str: password, or if hash is used empty string
    """
    if check_user_exists(username):
        raise UserAlreadyExistsException('User {} already exists'.format(username))

    # The username does not exist, let's create it
    if not properties:
        properties = dict()

    if 'password' in properties and password and (password != properties['password']):
        raise ValueError("Password argument is different than 'password' property")

    # decide returned password value and request body password according to arguments
    if 'hash' in properties:
        password = ''  # password is not effective, return empty string
    elif 'password' in properties:
        password = properties['password']  # return the effective password
    else:
        password = password or password_generator()
        properties['password'] = password

    create_sg_user = requests.put('{}/internalusers/{}'.format(settings.SEARCHGUARD_API_URL, username),
                                  data=json.dumps(properties), headers=settings.HEADER, auth=settings.SEARCHGUARD_API_AUTH)

    if create_sg_user.status_code == 201:
        # User created successfully
        return password
    else:
        # Raise exception because we received an error when creating the user
        raise CreateUserException('Error creating user {} - msg: {}'.format(username, create_sg_user.text))


def modify_user(user, properties):
    """Modifies a Search Guard user. Returns when successfully modified"""
    if check_user_exists(user):
        # The user does exist, let's modify it
        modify_sg_user = requests.put('{}/internalusers/{}'.format(settings.SEARCHGUARD_API_URL, user),
                                      data=json.dumps(properties), headers=settings.HEADER, auth=settings.SEARCHGUARD_API_AUTH)

        if modify_sg_user.status_code == 200:
            # User modified successfully
            return
        else:
            # Raise exception because we received an error when modifying the user
            raise ModifyUserException('Error modifying user {} - msg: {}'.format(user, modify_sg_user.text))
    else:
        # Raise exception because the user does not exist
        raise ModifyUserException('User {} does not exist'.format(user))


def delete_user(username):
    """Deletes a Search Guard user. Returns when successfully deleted"""
    if not check_user_exists(username):
        # Raise exception because the user does not exist
        raise DeleteUserException('Error deleting the user {}, does not exist'.format(username))

    # The user does exist, let's delete it
    delete_sg_user = requests.delete('{}/internalusers/{}'.format(settings.SEARCHGUARD_API_URL, username), auth=settings.SEARCHGUARD_API_AUTH)

    if delete_sg_user.status_code != 200:
        # Raise exception because we could not delete the user
        raise DeleteUserException('Error deleting the user {} - msg: {}'.format(username, delete_sg_user.text))


def view_user(user):
    """Returns information about a Search Guard user (when the user exists)
    The returned information contains the password hash and if present an array of the user's backend roles.
    :param str user: Name of the user to view in Search Guard
    """
    if check_user_exists(user):
        # The user does exist, let's view it
        view_sg_user = requests.get('{}/internalusers/{}'.format(settings.SEARCHGUARD_API_URL, user), auth=settings.SEARCHGUARD_API_AUTH)

        if view_sg_user.status_code == 200:
            return view_sg_user.text
        else:
            # Raise exception because we could not view the user
            raise ViewUserException('Error viewing the user {} - msg {}'.format(user, view_sg_user.text))
    else:
        # Raise exception because the user does not exist
        raise ViewUserException('Error viewing the user {}, does not exist'.format(user))


def list_users(prefix=None, search=None):
    """Returns the existing Search Guard users that match the filter criteria
    When no filters are specified, all users are returned.
    :param str prefix: Return only users that match this prefix (underscore is used as delimiter)
    :param str search: Return only users that contain this search string
    """
    response = requests.get('{}/internalusers/'.format(settings.SEARCHGUARD_API_URL), auth=settings.SEARCHGUARD_API_AUTH)

    if response.status_code == 200:
        list_sg_users = json.loads(response.text)
        # The API returned a list of existing users
        if prefix and search:
            # Return list of users filtered on prefix and search string
            return {k: v for (k, v) in list_sg_users.items() if k.split('_')[0] == prefix and search in k}
        if prefix:
            # Return list of users filtered on prefix
            return {k: v for (k, v) in list_sg_users.items() if k.split('_')[0] == prefix}
        if search:
            # Return list of users filtered on search string
            return {k: v for (k, v) in list_sg_users.items() if search in k}

        # Return all existing users (unfiltered)
        return list_sg_users
    else:
        # Raise exception because the API did not return code 200
        raise ListUsersException('Error listing users. status: {} - body: {}'.format(response.status_code, response.text))
